@@ -457,16 +457,32 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 				(char *) memmem(videotag, remainder, "src",
 						3);
 			    if (videosrc != NULL) {
-				videosrc = strchr(videosrc, '\"');
-				if (videosrc == NULL)
+				if (strchr(videosrc, '\"') == NULL)
+				    videosrc = strchr(videosrc, '\'');
+				else
+				    videosrc = strchr(videosrc, '\"');
+
+				if (videosrc == NULL) {
+				    seqtag = (char *) NULL;
+				    p = NULL;
+				    videotag = NULL;
 				    break;
+				}
 				i = (long) videosrc - (long) buffer_lower;
 				sscanf(buffer + i, "\"%1023[^\"]\"", url);
+				if (url == NULL)
+				    sscanf(buffer + i, "\'%1023[^\']\'",
+					   url);
 				if (DEBUG)
 				    printf("[smil] found url %s\n", url);
 				parent->playlist = 1;
 				addToList(instance, url, parent, speed);
 				found = 1;
+			    } else {
+				seqtag = (char *) NULL;
+				p = NULL;
+				videotag = NULL;
+				break;
 			    }
 			    find_area_tags(videosrc, instance->td->list);
 
@@ -490,6 +506,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 				videotag = NULL;
 			}
 			p = (char *) seqtag;
+		    } else {
+			p = (char *) seqtag + 4;	// for the seq block that contains no video or audio tags
 		    }
 		}
 	    }
@@ -611,8 +629,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			    *sub = '\0';
 			    if (DEBUG)
 				printf("URL: %s\n", url);
-			    parent->playlist = 1;
 			    addToList(instance, url, parent, -1);
+			    parent->playlist = 1;
 			    if (parent->mmsstream == 1
 				&& parent->next == NULL)
 				parent->playlist = 0;
