@@ -7,6 +7,7 @@ int enable_real;
 int enable_qt;
 int enable_wm;
 int enable_mpeg;
+int enable_ogg;
 
 #define MAX_BUF_LEN 255
 #define STATE_RESET 0
@@ -49,6 +50,7 @@ char *GetMIMEDescription()
     enable_qt = 1;
     enable_wm = 1;
     enable_mpeg = 1;
+    enable_ogg = 1;
     use_custom_mime_types = 0;
 
     DEBUG = 0;			//set to 1 when debugging this function
@@ -120,7 +122,15 @@ char *GetMIMEDescription()
 		    printf("mpeg:%i\n", enable_mpeg);
 		continue;
 	    }
-
+	    // Ogg Vorbis
+	    if (strncasecmp(buffer, "enable-ogg", 11) == 0) {
+		snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		sscanf(parse, "%i", &enable_ogg);
+		if (DEBUG)
+		    printf("mpeg:%i\n", enable_ogg);
+		continue;
+	    }
 	    if (strncasecmp(buffer, "use-mimetypes", 13) == 0) {
 		snprintf(parse, 1000, "%s", strtok(buffer, "="));
 		snprintf(parse, 1000, "%s", strtok(NULL, "="));
@@ -184,7 +194,7 @@ char *GetMIMEDescription()
 		    "application/x-mplayer2:wmv:WMV;"
 		    "video/x-ms-wm:asf:MSNBCPlayer;"
 		    "video/x-ms-asf:asf,asx:Windows Media;"
-		    "video/x-ms-wmv:wmv:Windows Media;"
+		    "video/x-ms-wmv:wmv:Microsoft WMV video;"
 		    "video/x-ms-wmp:wmp:Windows Media;"
 		    "video/x-ms-wvx:wvx:Windows Media;"
 		    "audio/x-ms-wax:wax:Windows Media;"
@@ -204,6 +214,11 @@ char *GetMIMEDescription()
 		    "audio/x-realaudio:ra:RealAudio;", sizeof(MimeTypes));
 	}
 
+	if (enable_real == 0 && enable_qt == 1) {
+	    strlcat(MimeTypes,
+		    "application/smil:smil:SMIL;", sizeof(MimeTypes));
+	}
+
 	if (enable_mpeg == 1) {
 	    strlcat(MimeTypes,
 		    "video/mpeg:mpg,mpeg:MPEG;"
@@ -219,6 +234,14 @@ char *GetMIMEDescription()
 		    "audio/mp3:mp3:MPEG audio;"
 		    "video/mp4:mp4:MPEG 4 Video;", sizeof(MimeTypes));
 	}
+
+	if (enable_ogg == 1) {
+	    strlcat(MimeTypes,
+		    "application/x-ogg:ogg:Ogg Vorbis Media;"
+		    "audio/ogg:ogg:Ogg Vorbis Audio;"
+		    "application/ogg:ogg:Ogg Vorbis / Ogg Theora;",
+		    sizeof(MimeTypes));
+	}
 	// FLI
 	strlcat(MimeTypes,
 		"video/fli:fli,flc:FLI animation;"
@@ -329,11 +352,25 @@ NPError GetValue(NPPVariable variable, void *value)
 
     }
     if (variable == NPPVpluginDescriptionString) {
+#ifdef GTK2_ENABLED
 	*((const char **) value) =
 	    "<a href=\"http://mplayerplug-in.sourceforge.net/\">mplayerplug-in</a> "
 	    PACKAGE_VERSION
-	    "<br><br>Video Player Plug-in for QuickTime, RealPlayer and Windows Media Player streams using <a href=\"http://mplayerhq.hu\">MPlayer</a> <br>"
-	    "JavaScript Enabled<br>";
+	    "<br><br>Video Player Plug-in for QuickTime, RealPlayer and Windows Media Player streams using <a href=\"http://mplayerhq.hu\">MPlayer</a> <br>JavaScript Enabled and Using GTK2 Widgets<br>";
+#endif
+#ifdef GTK1_ENABLED
+	*((const char **) value) =
+	    "<a href=\"http://mplayerplug-in.sourceforge.net/\">mplayerplug-in</a> "
+	    PACKAGE_VERSION
+	    "<br><br>Video Player Plug-in for QuickTime, RealPlayer and Windows Media Player streams using <a href=\"http://mplayerhq.hu\">MPlayer</a> <br>JavaScript Enabled and Using GTK1 Widgets<br>";
+#endif
+#ifdef X_ENABLED
+	*((const char **) value) =
+	    "<a href=\"http://mplayerplug-in.sourceforge.net/\">mplayerplug-in</a> "
+	    PACKAGE_VERSION
+	    "<br><br>Video Player Plug-in for QuickTime, RealPlayer and Windows Media Player streams using <a href=\"http://mplayerhq.hu\">MPlayer</a> <br>JavaScript Enabled and Using X Widgets<br>";
+#endif
+
     }
     if ((variable != NPPVpluginNameString)
 	&& (variable != NPPVpluginDescriptionString)) {
@@ -351,8 +388,12 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
     char parse[1000];
     char *cp;
 
+    if (DEBUG)
+	printf("mimetype: %s\n", parameters->type);
+
     instance->mode = parameters->mode;
     instance->mInstance = parameters->instance;
+    instance->mimetype = strdup(parameters->type);
 
     if (parameters->mode == NP_EMBED) {
 
@@ -394,6 +435,24 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		sscanf(parameters->argv[i], "%i", &instance->embed_width);
 	    }
 
+	    if (strncasecmp(parameters->argn[i], "hidden", 6) == 0) {
+
+		lowercase(parameters->argv[i]);
+		if (DEBUG)
+		    printf("argv[i]=%s\n", parameters->argv[i]);
+
+		if (strstr(parameters->argv[i], "true")
+		    || strstr(parameters->argv[i], "yes")
+		    || strstr(parameters->argv[i], "1")) {
+		    instance->hidden = 1;
+		} else {
+		    instance->hidden = 0;
+		}
+		if (DEBUG)
+		    printf("hidden=%i\n", instance->hidden);
+	    }
+
+
 	    /* handle 'scale' attribute used by QT instance */
 	    if (strncasecmp(parameters->argn[i], "scale", 5) == 0) {
 		if (strncasecmp(parameters->argv[i], "aspect", 6) == 0) {
@@ -436,7 +495,9 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    printf("autostart=%i\n", instance->autostart);
 	    }
 
-	    if (strncasecmp(parameters->argn[i], "showcontrols", 11) == 0) {
+	    if ((strncasecmp(parameters->argn[i], "showcontrols", 11) == 0)
+		|| ((strncasecmp(parameters->argn[i], "controls", 8) == 0)
+		    && (strstr(instance->mimetype, "quicktime") != NULL))) {
 
 		lowercase(parameters->argv[i]);
 		if (DEBUG)
@@ -471,7 +532,8 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 	    }
 
 
-	    if (strncasecmp(parameters->argn[i], "controls", 8) == 0) {
+	    if ((strncasecmp(parameters->argn[i], "controls", 8) == 0)
+		&& (strstr(instance->mimetype, "quicktime") == NULL)) {
 
 		lowercase(parameters->argv[i]);
 		if (DEBUG)
@@ -482,6 +544,7 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    || strstr(parameters->argv[i], "1")
 		    || strstr(parameters->argv[i], "all")
 		    || strstr(parameters->argv[i], "statusfield")
+		    || strstr(parameters->argv[i], "statuspanel")
 		    || strstr(parameters->argv[i], "playbutton")
 		    || strstr(parameters->argv[i], "volumeslider")
 		    || strstr(parameters->argv[i], "stopbutton")
@@ -695,7 +758,7 @@ void LoadConfigFile(nsPluginInstance * instance)
 
 	    if (strncasecmp(buffer, "ao", 2) == 0) {
 		snprintf(parse, 1000, "%s", strtok(buffer, "="));
-		snprintf(parse, 1000, "%s", strtok(NULL, "=\n"));
+		snprintf(parse, 1000, "%s", strtok(NULL, "\n"));
 		instance->ao = strdup(parse);
 		continue;
 	    }
