@@ -11,7 +11,7 @@ extern int DEBUG;
 
 static void sig_child(int signo)
 {
-    wait(0L);
+    wait(NULL);
 }
 
 /*
@@ -117,6 +117,7 @@ void launchPlayerThread(nsPluginInstance * instance)
 	printf
 	    ("****WARNING: launching duplicate player thread, js_state = %d\n",
 	     instance->js_state);
+	instance->threadlaunched = 0;
     }
 }
 
@@ -128,6 +129,10 @@ void signalPlayerThread(nsPluginInstance * instance)
 	       instance->state, instance->js_state);
     }
 
+    if (instance->threadlaunched != 1)
+	if (DEBUG)
+	    printf("****Player thread did not launch correctly****\n");
+
     pthread_mutex_lock(&(instance->control_mutex));
     while (instance->js_state == JS_STATE_INITIALIZING) {
 	if (DEBUG)
@@ -220,6 +225,9 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     instance->td->w = instance->widget;
     instance->td->instance = instance;
 
+    if (DEBUG > 1)
+	printf("Entering SetupPlayer\n");
+
 #ifdef X_ENABLED
     DrawUI(instance->widget, instance, "Loading Media...", 0, -1);
 #endif
@@ -336,11 +344,11 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 #endif
 #ifdef GTK_ENABLED
 	    instance->drawing_area = gtk_drawing_area_new();
+	    gtk_widget_set_events(instance->drawing_area,
+				  GDK_BUTTON_PRESS_MASK);
 	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			  instance->drawing_area, 0, 0);
 	    gtk_widget_show(instance->drawing_area);
-	    gtk_widget_set_events(instance->drawing_area,
-				  GDK_BUTTON_PRESS_MASK);
 	    instance->player_window =
 		GDK_WINDOW_XWINDOW(instance->drawing_area->window);
 	    snprintf(buffer, 1024, "0x%x", (int) instance->player_window);
@@ -361,13 +369,13 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 #endif
 #ifdef GTK_ENABLED
 	    instance->drawing_area = gtk_drawing_area_new();
+	    gtk_widget_set_events(instance->drawing_area,
+				  GDK_BUTTON_PRESS_MASK);
 	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			  instance->drawing_area, 10, 100);
 	    gtk_widget_set_usize(GTK_WIDGET(instance->status),
 				 instance->window_width - 20, 19);
 	    gtk_widget_show(instance->drawing_area);
-	    gtk_widget_set_events(instance->drawing_area,
-				  GDK_BUTTON_PRESS_MASK);
 	    instance->player_window =
 		GDK_WINDOW_XWINDOW(instance->drawing_area->window);
 	    snprintf(buffer, 1024, "0x%x", (int) instance->player_window);
@@ -568,7 +576,6 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 {
 
     char buffer[1024];
-
     char message[1024];
     int notfound;
     char *vo;
@@ -583,7 +590,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
     int i;
     int zerocfbytes_count = 0;
     char url_copy[1024];
-
+    int c;
 
 #define TRYAGAIN_FALSE 0
 #define TRYAGAIN_TRUE 1
@@ -629,11 +636,41 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 #ifdef GTK_ENABLED
 	g_idle_add(gtkgui_save_enable, local_td->instance);
 #endif
-	if (fgets(buffer, 1024, local_td->instance->player) == NULL) {
-	    continue;
-	}
 
 	pthread_testcancel();
+	if (local_td->instance->player != NULL) {
+
+	    //if (fgets(buffer, 1024, local_td->instance->player) == NULL) {
+	    //  continue;
+	    //}
+
+	    // fgets is not a pthread cancel point, so we basically have to rewrite fgets
+	    // to make this work better. If we cancel the thread while fgets is waiting for 
+	    // for an EOS then we crash.
+	    buffer[0] = '\0';
+	    i = 0;
+	    do {
+		pthread_testcancel();
+		c = fgetc(local_td->instance->player);
+		if (c == EOF) {
+		    buffer[i] = '\0';
+		    break;
+		}
+		if (c == 0)
+		    continue;
+		buffer[i] = (char) c;
+		i++;
+		if (i >= 1024) {
+		    buffer[1023] = '\0';
+		    break;
+		}
+	    } while (((unsigned char) c != '\n'));
+	    if (buffer[0] == '\0')
+		continue;
+	    else
+		buffer[i] = '\0';	// make sure we NULL terminate the string
+	}
+	pthread_testcancel();
 
 	if (DEBUG) {
 	    printf("READ: %s \n", buffer);
@@ -701,7 +738,6 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		g_idle_add(gtkgui_message, local_td->instance);
 		local_td->instance->percent = (cfpercent / 100.0);
 		g_idle_add(gtkgui_progress, local_td->instance);
-
 #endif
 	    }
 	}
@@ -964,7 +1000,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 
 	assert(local_td->instance->player != NULL);
-
+/*
 	//may not be completely safe
 	pthread_mutex_lock(&(local_td->instance->playlist_mutex));
 	if (local_td->instance->cancelled == 1) {
@@ -976,11 +1012,12 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    break;
 	}
 	pthread_mutex_unlock(&(local_td->instance->playlist_mutex));
+*/
     }
 
 
     if (DEBUG) {
-	printf("---player thread: tryagain = %d\n", tryagain);
+	printf("----player thread: tryagain = %d\n", tryagain);
     }
     //we always return either true or false
     if (tryagain == TRYAGAIN_TRUE) {
@@ -1032,7 +1069,7 @@ void *playPlaylist(void *td)
     int usefps;
     int nomouseinput;
     int listempty;
-    
+
     local_td = (ThreadData *) td;
     local_list = local_td->list;
 
@@ -1185,20 +1222,21 @@ void *playPlaylist(void *td)
 	    local_mmsstream = 0;
 	} else {
 	    //we have a node to play
-	    if (DEBUG) 
-	        printf("----player thread: chose url %s\n", local_list->url);
+	    if (DEBUG)
+		printf("----player thread: chose url %s\n",
+		       local_list->url);
 	    listempty = 0;
 	    snprintf(local_url, 1024, "%s", local_list->url);
 	    local_url[1023] = '\0';
 	    local_mmsstream = local_list->mmsstream;
-        }
-	
+	}
+
 	pthread_mutex_unlock(&(local_td->instance->playlist_mutex));
 	pthread_cleanup_pop(0);
 
 	if (listempty == 1)
 	    break;
-	
+
 	pthread_testcancel();
 
 	usefps = 0;
@@ -1301,6 +1339,7 @@ void *playPlaylist(void *td)
 	    if (local_td->instance->player != NULL) {
 		local_td->instance->js_state = JS_STATE_PLAYING;
 	    }
+
 	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 	    pthread_cleanup_pop(0);
 
@@ -1409,6 +1448,12 @@ void *playPlaylist(void *td)
 
     pthread_mutex_lock(&(local_td->instance->control_mutex));
     local_td->instance->js_state = JS_STATE_UNDEFINED;
+    argc = base_argc;
+    while (argc < 50) {
+	if (local_td->argv[argc] != NULL)
+	    free(local_td->argv[argc]);
+	local_td->argv[argc++] = NULL;
+    }
     pthread_mutex_unlock(&(local_td->instance->control_mutex));
 
     //once we get here, the thread is only allowed to access stack
@@ -1418,6 +1463,8 @@ void *playPlaylist(void *td)
     if (DEBUG) {
 	printf("----player thread: normal exit\n");
     }
+
+
     pthread_exit(0);
 
     return NULL;
