@@ -241,7 +241,7 @@ void DrawUI(Widget w, nsPluginInstance * instance, char *message,
     int PercentRedraw;
 
     if (0) {
-	printf("Widget: %i\n", (int)w);
+	printf("Widget: %i\n", (int) w);
 	printf("Message: %s\n", message);
     }
 
@@ -1039,8 +1039,10 @@ gboolean gtkgui_message(void *data)
 	return FALSE;
     if (instance->mInitialized == FALSE)
 	return FALSE;
+
     if (instance->status != NULL && instance->lastmessage != NULL)
-	gtk_label_set_text(instance->status, instance->lastmessage);
+	if (GTK_IS_LABEL(instance->status))
+	    gtk_label_set_text(instance->status, instance->lastmessage);	//GTK_LABEL
     return FALSE;
 }
 
@@ -1055,14 +1057,17 @@ gboolean gtkgui_progress(void *data)
     if (instance == NULL)
 	return FALSE;
     if (instance->progress_bar != NULL) {
-	if ((instance->percent > 0.0) && (instance->percent < 1.0)) {
-	    if (instance->movie_height == 0 && instance->movie_width == 0)
-		gtk_widget_show(GTK_WIDGET(instance->progress_bar));
-	    gtk_progress_bar_update(instance->progress_bar,
-				    instance->percent);
+	if (GTK_IS_PROGRESS_BAR(instance->progress_bar)) {
+	    if ((instance->percent > 0.0) && (instance->percent < 1.0)) {
+		if (instance->movie_height == 0
+		    && instance->movie_width == 0)
+		    gtk_widget_show(GTK_WIDGET(instance->progress_bar));
+		gtk_progress_bar_update(instance->progress_bar,
+					instance->percent);
+	    }
+	    if (instance->percent >= 0.99 || instance->percent == 0)
+		gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
 	}
-	if (instance->percent >= 0.99 || instance->percent == 0)
-	    gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
     }
     return FALSE;
 }
@@ -1081,17 +1086,22 @@ gboolean gtkgui_resize(void *data)
 	printf("in resize method\nNew size is %i x %i\n",
 	       instance->movie_width, instance->movie_height);
     if (instance->drawing_area != NULL) {
-	gtk_widget_set_usize(instance->drawing_area,
-			     instance->movie_width,
-			     instance->movie_height);
-	gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
-	if (instance->mode == NP_FULL)
-	    if ((instance->movie_width + 10 >= instance->window_width)
-		|| (instance->movie_height + 100 >=
-		    instance->window_height))
-		if (instance->fullscreen == 0)
-		    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
-				  instance->drawing_area, 0, 0);
+	if (GTK_IS_WIDGET(instance->drawing_area)) {
+	    gtk_widget_set_usize(instance->drawing_area,
+				 instance->movie_width,
+				 instance->movie_height);
+	    if (GTK_IS_WIDGET(instance->progress_bar))
+		gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
+	    if (instance->mode == NP_FULL)
+		if ((instance->movie_width + 10 >= instance->window_width)
+		    || (instance->movie_height + 100 >=
+			instance->window_height))
+		    if (instance->fullscreen == 0)
+			if (GTK_IS_CONTAINER(instance->fixed_container))
+			    gtk_fixed_put(GTK_FIXED
+					  (instance->fixed_container),
+					  instance->drawing_area, 0, 0);
+	}
     }
     return FALSE;
 
@@ -1109,6 +1119,8 @@ gboolean gtkgui_stop(void *data)
 	return FALSE;
     if (instance->mInitialized == FALSE)
 	return FALSE;
+    if (instance->cancelled == 1)
+	return FALSE;
     stop_callback(NULL, NULL, instance);
     instance->SetFullscreen(0);
     if (DEBUG > 1)
@@ -1154,16 +1166,26 @@ gboolean gtkgui_save_enable(void *data)
 	    snprintf(newlabel, 1024, "Save");
 	}
 
-	label =
-	    GTK_LABEL(gtk_bin_get_child(GTK_BIN(instance->menuitem_save)));
-	gtk_label_set_text(label, newlabel);
+	if (GTK_IS_BIN(instance->menuitem_save)) {
+	    label =
+		GTK_LABEL(gtk_bin_get_child
+			  (GTK_BIN(instance->menuitem_save)));
+	    gtk_label_set_text(label, newlabel);
+	}
 #endif
 	pthread_mutex_unlock(&(instance->playlist_mutex));
 	gtk_widget_set_sensitive(GTK_WIDGET(instance->menuitem_save),
 				 instance->currentnode->retrieved);
     } else {
+#ifdef GTK2_ENABLED
+	snprintf(newlabel, 1024, "Append URL to %s/playlist",
+		 instance->download_dir);
+	label =
+	    GTK_LABEL(gtk_bin_get_child(GTK_BIN(instance->menuitem_save)));
+	gtk_label_set_text(label, newlabel);
+#endif
 	gtk_widget_set_sensitive(GTK_WIDGET(instance->menuitem_save),
-				 FALSE);
+				 TRUE);
     }
     return FALSE;
 }
@@ -1182,14 +1204,17 @@ gboolean gtkgui_drawMediaProgress(void *data)
     if (instance->mInitialized == FALSE)
 	return FALSE;
     if (instance->mediaprogress_bar != NULL) {
-	if ((instance->mediaPercent > 0)
-	    && (instance->mediaPercent <= 100)) {
-	    if (instance->controlsvisible)
-		gtk_widget_show(GTK_WIDGET(instance->mediaprogress_bar));
-	    gtk_progress_bar_update(instance->mediaprogress_bar,
-				    instance->mediaPercent / 100.0);
-	} else {
-	    gtk_widget_hide(GTK_WIDGET(instance->mediaprogress_bar));
+	if (GTK_IS_PROGRESS_BAR(instance->mediaprogress_bar)) {
+	    if ((instance->mediaPercent > 0)
+		&& (instance->mediaPercent <= 100)) {
+		if (instance->controlsvisible)
+		    gtk_widget_show(GTK_WIDGET
+				    (instance->mediaprogress_bar));
+		gtk_progress_bar_update(instance->mediaprogress_bar,
+					instance->mediaPercent / 100.0);
+	    } else {
+		gtk_widget_hide(GTK_WIDGET(instance->mediaprogress_bar));
+	    }
 	}
     }
 
@@ -1339,13 +1364,19 @@ void menuitem_fullscreen_callback(GtkCheckMenuItem * menuitem,
 
 }
 
-void menuitem_save_callback(GtkCheckMenuItem * menuitem,
+void menuitem_save_callback(GtkMenuItem * menuitem,
 			    nsPluginInstance * instance)
 {
     char *filename;
     char newname[1024];
+    char mmsplaylist[1024];
+    FILE *playlist;
+    FILE *fin, *fout;
+    char buffer[1000];
+    int count;
     Node *n, *lastplayed;
 
+
     if (instance->mInitialized == FALSE)
 	return;
     pthread_mutex_lock(&(instance->playlist_mutex));
@@ -1369,17 +1400,72 @@ void menuitem_save_callback(GtkCheckMenuItem * menuitem,
 	    if (lastplayed->retrieved == 1) {
 		snprintf(newname, 1024, "%s/%s",
 			 instance->download_dir, filename);
-		rename(lastplayed->fname, newname);
-		snprintf(lastplayed->fname, 1024, "%s/%s",
-			 instance->download_dir, filename);
-		lastplayed->remove = 0;
+		if (rename(lastplayed->fname, newname) == 0) {
+		    snprintf(lastplayed->fname, 1024, "%s/%s",
+			     instance->download_dir, filename);
+		    lastplayed->remove = 0;
+		} else {
+		    // rename failed (probably cross partitions), copy the file
+		    fin = fopen(lastplayed->fname, "rb");
+		    fout = fopen(newname, "wb");
+		    if (fin != NULL && fout != NULL) {
+			while (!feof(fin)) {
+			    count = fread(buffer, 1, 1000, fin);
+			    fwrite(buffer, 1, count, fout);
+			}
+			fclose(fout);
+			fclose(fin);
+		    }
+		}
 	    }
 	    NPN_MemFree(filename);
 	}
+	if (lastplayed->mmsstream) {
+	    snprintf(mmsplaylist, sizeof(mmsplaylist),
+		     "%s/playlist", instance->download_dir);
+	    playlist = fopen(mmsplaylist, "a");
+	    if (playlist != NULL) {
+		fprintf(playlist, "%s\n", instance->list->url);
+		fclose(playlist);
+	    }
+	}
     }
     pthread_mutex_unlock(&(instance->playlist_mutex));
 }
 
+void menuitem_copy_callback(GtkMenuItem * menuitem,
+			    nsPluginInstance * instance)
+{
+    Node *n, *lastplayed;
+#ifdef GTK2_ENABLED
+    GtkClipboard *clipboard;
+#endif
+    char localurl[1024];
+
+    lastplayed = NULL;
+
+    if (instance->currentnode != NULL) {
+	lastplayed = instance->currentnode;
+    } else {
+	n = instance->list;
+	while (n != NULL) {
+	    if (n->played == 1)
+		lastplayed = n;
+	    n = n->next;
+	}
+    }
+
+    if (lastplayed != NULL) {
+	fullyQualifyURL(instance, lastplayed->url, localurl);
+#ifdef GTK2_ENABLED
+	clipboard = gtk_clipboard_get(GDK_NONE);
+	gtk_clipboard_set_text(clipboard, localurl, strlen(localurl));
+#endif
+#ifdef GTK1_ENABLED
+	XStoreBytes(instance->display, localurl, strlen(localurl));
+#endif
+    }
+}
 
 
 #endif
