@@ -209,6 +209,9 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     fixed_container = NULL;
     drawing_area = NULL;
     popup_menu = NULL;
+#ifdef GTK2_ENABLED
+    fs_window = NULL;
+#endif
 #endif
 
 #ifdef X_ENABLED
@@ -266,20 +269,23 @@ nsPluginInstance::~nsPluginInstance()
     // and zero its mPlugin member
     if (DEBUG)
 	printf("~nsPluginInstance called\n");
+#ifdef GTK_ENABLED
+    gdk_flush();
+#endif
 
+    mInstance = NULL;
+    mInitialized = FALSE;
     if (mControlsScriptablePeer != NULL) {
 	mControlsScriptablePeer->SetInstance(NULL);
 	mControlsScriptablePeer->Release();
 	NS_IF_RELEASE(mControlsScriptablePeer);
     }
 
-
     if (mScriptablePeer != NULL) {
 	mScriptablePeer->InitControls(NULL);
 	mScriptablePeer->SetInstance(NULL);
 	NS_IF_RELEASE(mScriptablePeer);
     }
-
 }
 
 NPBool nsPluginInstance::init(NPWindow * aWindow)
@@ -310,7 +316,14 @@ void nsPluginInstance::shut()
 
     mInitialized = FALSE;
 
-#ifdef NOT_ENABLED		// probably causes a GTK memory leak, but stops weird crash and warnings
+#ifdef GTK_ENABLED
+    while (g_idle_remove_by_data(this)) {
+	if (DEBUG)
+	    printf("Removing function from idle handler\n");
+    }
+#endif
+
+#ifdef GTK2_ENABLED		// When disabled probably causes a GTK memory leak, but stops weird crash and warnings
     if (controlwindow == 0) {
 	if (panel_drawn == 1) {
 	    gtk_widget_destroy(play_event_box);
@@ -344,16 +357,74 @@ void nsPluginInstance::shut()
 	    gtk_widget_destroy(GTK_WIDGET(menuitem_sep2));
 	    gtk_widget_destroy(GTK_WIDGET(menuitem_showcontrols));
 	    gtk_widget_destroy(GTK_WIDGET(menuitem_fullscreen));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_copy));
 	    gtk_widget_destroy(GTK_WIDGET(menuitem_save));
 	    gtk_widget_destroy(GTK_WIDGET(popup_menu));
 	}
 
 	if (gtkwidget != NULL)
 	    gtk_widget_destroy(gtkwidget);
-	//gdk_window_process_all_updates();
     }
 #endif
 
+#ifdef GTK1_ENABLED
+    if (controlwindow == 0) {
+
+	if (popup_menu != NULL) {
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_play));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_pause));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_stop));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_sep1));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_sep2));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_showcontrols));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_fullscreen));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_copy));
+	    gtk_widget_destroy(GTK_WIDGET(menuitem_save));
+	    gtk_widget_destroy(GTK_WIDGET(popup_menu));
+	}
+	if (GTK_IS_WIDGET(gtkwidget))
+	    gtk_widget_destroy(gtkwidget);
+    }
+    gdk_pixmap_unref(logo);
+    gdk_pixmap_unref(pb_sm_play_up);
+    gdk_pixmap_unref(pb_sm_play_down);
+    gdk_pixmap_unref(pb_sm_pause_up);
+    gdk_pixmap_unref(pb_sm_pause_down);
+    gdk_pixmap_unref(pb_sm_stop_up);
+    gdk_pixmap_unref(pb_sm_stop_down);
+    gdk_pixmap_unref(pb_sm_ff_up);
+    gdk_pixmap_unref(pb_sm_ff_down);
+    gdk_pixmap_unref(pb_sm_rew_up);
+    gdk_pixmap_unref(pb_sm_rew_down);
+    gdk_pixmap_unref(pb_sm_fs_up);
+    gdk_pixmap_unref(pb_sm_fs_down);
+
+    gdk_flush();
+    XSync(display, FALSE);
+
+#endif
+
+#ifdef GTK_ENABLED
+    image = NULL;
+    progress_bar = NULL;
+    mediaprogress_bar = NULL;
+    status = NULL;
+    play_event_box = NULL;
+    pause_event_box = NULL;
+    stop_event_box = NULL;
+    ff_event_box = NULL;
+    rew_event_box = NULL;
+    fs_event_box = NULL;
+    gtkwidget = NULL;
+    fixed_container = NULL;
+    drawing_area = NULL;
+    popup_menu = NULL;
+#ifdef GTK2_ENABLED
+    fs_window = NULL;
+#endif
+    gtkwidget = NULL;
+#endif
+
     if (mimetype != NULL) {
 	free(mimetype);
 	mimetype = NULL;
@@ -447,11 +518,13 @@ void nsPluginInstance::shut()
     mmsstream = 0;
     cancelled = 0;
     js_state = JS_STATE_UNDEFINED;
+
     pthread_attr_destroy(&thread_attr);
     pthread_mutex_destroy(&playlist_mutex);
     pthread_mutex_destroy(&playlist_cond_mutex);
     pthread_mutex_destroy(&control_mutex);
     pthread_cond_destroy(&playlist_complete_cond);
+
 }
 
 NPBool nsPluginInstance::isInitialized()
@@ -469,7 +542,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
     int multiplier, height, width;
 #endif
 #ifdef GTK1_ENABLED
-    GtkStyle *s;
+//    GtkStyle *s;
 #endif
 
     if (DEBUG >= 2)
@@ -484,7 +557,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
     if ((Window) window != (Window) aWindow->window) {
 	if (DEBUG)
 	    printf("New window! old: 0x%x    new 0x%x\n",
-		   (unsigned int)window, (unsigned int)aWindow->window);
+		   (unsigned int) window, (unsigned int) aWindow->window);
     }
 
     if (controlwindow == 1)
@@ -509,7 +582,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
     if (state < STATE_WINDOWSET) {
 	if (DEBUG) {
 	    printf("Size: %d %d %x\n", aWindow->x, aWindow->y,
-		   (unsigned int)aWindow->window);
+		   (unsigned int) aWindow->window);
 	    printf("Size: %dx%d \n", aWindow->width, aWindow->height);
 	}
 
@@ -532,9 +605,17 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	state = STATE_WINDOWSET;
 
 #ifdef GTK_ENABLED
-
+#ifdef GTK2_ENABLED
 	gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
-
+#endif
+#ifdef GTK1_ENABLED
+	gdk_flush();
+	XSync(display, FALSE);
+	gtkwidget = gtk_plug_new(window);
+	gtk_widget_realize(gtkwidget);
+#endif
+	gdk_flush();
+	gtk_widget_set_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
 #ifdef GTK2_ENABLED
 	gtk_widget_set_size_request(gtkwidget, aWindow->width,
 				    aWindow->height);
@@ -543,11 +624,9 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 
 #ifdef GTK1_ENABLED
+	if (DEBUG > 1)
+	    printf("setting window size\n");
 	gtk_widget_set_usize(gtkwidget, aWindow->width, aWindow->height);
-	s = gtk_widget_get_style(gtkwidget);
-	color.red = s->bg[GTK_STATE_NORMAL].red;
-	color.green = s->bg[GTK_STATE_NORMAL].green;
-	color.blue = s->bg[GTK_STATE_NORMAL].blue;
 
 #endif
 
@@ -593,12 +672,15 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 	gtk_menu_append(popup_menu, GTK_WIDGET(menuitem_sep2));
 	gtk_widget_show(GTK_WIDGET(menuitem_sep2));
+	menuitem_copy =
+	    GTK_MENU_ITEM(gtk_menu_item_new_with_label("Copy URL"));
+	gtk_menu_append(popup_menu, GTK_WIDGET(menuitem_copy));
+	gtk_widget_show(GTK_WIDGET(menuitem_copy));
 	menuitem_save =
 	    GTK_MENU_ITEM(gtk_menu_item_new_with_label("Save"));
 	gtk_menu_append(popup_menu, GTK_WIDGET(menuitem_save));
 	gtk_widget_show(GTK_WIDGET(menuitem_save));
 	gtk_widget_set_sensitive(GTK_WIDGET(menuitem_save), FALSE);
-	gtk_widget_set_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
 
 #ifdef GTK2_ENABLED
 	g_signal_connect_swapped(GTK_OBJECT(gtkwidget),
@@ -617,6 +699,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			 G_CALLBACK(menuitem_fullscreen_callback), this);
 	g_signal_connect(GTK_OBJECT(menuitem_save), "activate",
 			 G_CALLBACK(menuitem_save_callback), this);
+	g_signal_connect(GTK_OBJECT(menuitem_copy), "activate",
+			 G_CALLBACK(menuitem_copy_callback), this);
 
 #endif
 
@@ -639,8 +723,12 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			   this);
 	gtk_signal_connect(GTK_OBJECT(menuitem_save), "activate",
 			   GTK_SIGNAL_FUNC(menuitem_save_callback), this);
+	gtk_signal_connect(GTK_OBJECT(menuitem_copy), "activate",
+			   GTK_SIGNAL_FUNC(menuitem_copy_callback), this);
 
 #endif
+	if (DEBUG > 1)
+	    printf("menus built\n");
 
 	fixed_container = gtk_fixed_new();
 	status = GTK_LABEL(gtk_label_new("Initializing"));
@@ -657,7 +745,14 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 					aWindow->height - 125);
 #endif
 #ifdef GTK1_ENABLED
+	    if (DEBUG > 1)
+		printf("> 125 setup\n");
 	    gtk_widget_show(gtkwidget);
+//          s = gtk_widget_get_style(gtkwidget);
+//          color.red = s->bg[GTK_STATE_NORMAL].red;
+//          color.green = s->bg[GTK_STATE_NORMAL].green;
+//          color.blue = s->bg[GTK_STATE_NORMAL].blue;
+//          gdk_flush();
 	    logo = gdk_pixmap_create_from_xpm_d(gtkwidget->window,
 						NULL, NULL, logo_xpm);
 	    image = gtk_pixmap_new(logo, NULL);
@@ -692,6 +787,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 					aWindow->height - 25);
 #endif
 #ifdef GTK1_ENABLED
+	    if (DEBUG > 1)
+		printf("< 125 setup\n");
 	    gtk_widget_set_usize(GTK_WIDGET(progress_bar),
 				 aWindow->width - 20, 15);
 	    gtk_widget_set_usize(GTK_WIDGET(status),
@@ -706,13 +803,19 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	gtk_widget_show(GTK_WIDGET(status));
 	gtk_widget_show(fixed_container);
 	gtk_widget_show(gtkwidget);
+	gdk_flush();
 	InitPixbufs(this);
-
+	if (DEBUG > 1)
+	    printf("all setup and ready to reparent and map\n");
+#ifdef GTK2_ENABLED
 	XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 			GDK_WINDOW_XWINDOW(gtkwidget->window), window, 0,
 			0);
-	XMapWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-		   GDK_WINDOW_XWINDOW(gtkwidget->window));
+	gtk_widget_map(gtkwidget);
+#endif
+	if (DEBUG > 1)
+	    printf("reparent and map done\n");
+
 #endif
 
     } else {
@@ -756,25 +859,35 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	}
     }
 
-    if (DEBUG > 1)
+    if (DEBUG)
 	printf("resizing widgets to %i x %i \n", aWindow->width,
 	       aWindow->height);
 
+#ifdef GTK_ENABLED
+    if (status != NULL) {
+	gtk_widget_show(GTK_WIDGET(status));
 #ifdef GTK2_ENABLED
-    gtk_widget_set_size_request(GTK_WIDGET(status),
-				aWindow->width - 20, aWindow->height - 25);
+	gtk_widget_set_size_request(GTK_WIDGET(status),
+				    aWindow->width - 20,
+				    aWindow->height - 25);
 #endif
 #ifdef GTK1_ENABLED
-    gtk_widget_set_usize(GTK_WIDGET(status),
-			 aWindow->width - 20, aWindow->height - 25);
+	gtk_widget_set_usize(GTK_WIDGET(status),
+			     aWindow->width - 20, aWindow->height - 25);
 #endif
+    }
+    if (gtkwidget == NULL)
+	return NPERR_NO_ERROR;
 
-#ifdef GTK_ENABLED
+    gtk_widget_show(gtkwidget);
     XResizeWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 		  GDK_WINDOW_XWINDOW(gtkwidget->window),
 		  aWindow->width, aWindow->height);
     XResizeWindow(display, window, aWindow->width, aWindow->height);
 
+    if (DEBUG > 1)
+	printf("resize is complete\n");
+
     if (panel_drawn == 1 && mode == NP_FULL) {
 	if (panel_height >= 32) {
 	    height = 32;
@@ -856,7 +969,6 @@ NPError nsPluginInstance::NewStream(NPMIMEType type, NPStream * stream,
     if (DEBUG >= 2)
 	printf("*********Exiting NewStream Callback*****************\n");
 
-
     return NPERR_NO_ERROR;
 }
 
@@ -870,8 +982,6 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 	printf("***********NPP_DestroyStream called %i\n URL: %s\n",
 	       reason, stream->url);
 
-
-
     if (reason == NPRES_DONE) {
 	playable = 0;
 	if (strlen(stream->url) >= 1023) {
@@ -1287,9 +1397,11 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 			gtk_widget_queue_draw(GTK_WIDGET(progress_bar));
 		    }
 		    if (status != NULL) {
-			gtk_label_set_text(status, message);
-			gtk_widget_show(GTK_WIDGET(status));
-			gtk_widget_queue_draw(GTK_WIDGET(status));
+			if (state < STATE_PLAYING) {
+			    gtk_label_set_text(status, message);
+			    gtk_widget_show(GTK_WIDGET(status));
+			    gtk_widget_queue_draw(GTK_WIDGET(status));
+			}
 		    }
 #endif
 		}
@@ -1407,7 +1519,7 @@ void nsPluginInstance::Play()
 {
     Node *n;
 
-    if (DEBUG >= 2) {
+    if (DEBUG > 1) {
 	printf("*****Play Called\n");
     }
 
@@ -1422,6 +1534,9 @@ void nsPluginInstance::Play()
 
     if (js_state == JS_STATE_UNDEFINED) {
 	//reset the playlist
+	if (DEBUG > 1)
+	    printf("Play: resetting playlist\n");
+
 	pthread_mutex_lock(&playlist_mutex);
 	n = list;
 	while (n != NULL) {
@@ -1433,6 +1548,8 @@ void nsPluginInstance::Play()
     }
 
     if (threadsetup == 0 && controlwindow == 0) {
+	if (DEBUG > 1)
+	    printf("Play: setupplayer\n");
 	state = STATE_GETTING_PLAYLIST;
 	pthread_mutex_unlock(&control_mutex);
 	SetupPlayer(this, NULL);
@@ -1440,19 +1557,26 @@ void nsPluginInstance::Play()
     }
 
     if (threadsignaled == 1 && js_state == JS_STATE_UNDEFINED) {
+	if (DEBUG > 1)
+	    printf("Play: launching thread\n");
+	state = STATE_NEWINSTANCE;
 	launchPlayerThread(this);
 	pthread_mutex_unlock(&control_mutex);
 	// recommended slight pause
 	usleep(1);
 	//signal player thread
+	if (DEBUG > 1)
+	    printf("Play: signaling thread after launch\n");
 	signalPlayerThread(this);
 	threadsignaled = 1;
 
     } else if ((autostart == 0) && (threadsignaled == 0)) {
 	pthread_mutex_unlock(&control_mutex);
+	if (DEBUG > 1)
+	    printf("Play: signaling thread without launch\n");
 
 	signalPlayerThread(this);
-
+	threadsignaled = 1;
     } else {
 	pthread_mutex_unlock(&control_mutex);
     }
@@ -1461,7 +1585,7 @@ void nsPluginInstance::Play()
     play_callback(NULL, NULL, this);
 #endif
 
-    if (DEBUG >= 2) {
+    if (DEBUG > 1) {
 	printf("***********Exiting Play*************\n");
     }
 
@@ -1713,10 +1837,10 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 {
 
 #ifdef GTK2_ENABLED
-    Display *xdisp;
     GdkDisplay *gdisp;
     GdkScreen *screen;
     GdkColor black;
+    GdkRectangle rect;
 #endif
 #ifdef GTK1_ENABLED
     GdkColor black;
@@ -1724,7 +1848,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 
 #ifdef GTK_ENABLED
     int multiplier, height = 0, width;
-    int x, y, disp_y, pos_y;
+    int x, y, disp_x, pos_x, disp_y, pos_y;
 #endif
     int win_height, win_width;
 
@@ -1745,6 +1869,12 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	win_width = window_width;
     }
 
+    if (DEBUG)
+	printf("height = %i and width = %i\n", win_height, win_width);
+
+    if (win_height == 0 || win_width == 0 || hidden == 1)
+	return;
+
     if (fullscreen) {
 	if (value) {
 	    // do nothing
@@ -1755,16 +1885,35 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 #ifdef GTK_ENABLED
 	    if (DEBUG > 1)
 		printf("SetFullscreen returning to original size\n");
-	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-			    GDK_WINDOW_XWINDOW(gtkwidget->window), window,
-			    0, 0);
-	    XMapWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-		       GDK_WINDOW_XWINDOW(gtkwidget->window));
-	    // need to resize here
+#ifdef GTK2_ENABLED
+	    if (fs_window != NULL) {
+		gtk_window_unfullscreen(GTK_WINDOW(fs_window));
+		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				GDK_WINDOW_XWINDOW(gtkwidget->window),
+				window, 0, 0);
+		gtk_widget_map(gtkwidget);
+		gtk_widget_destroy(fs_window);
+		fs_window = NULL;
+	    }
+	    gtk_window_move(GTK_WINDOW(gtkwidget), 0, 0);
+	    gtk_window_resize(GTK_WINDOW(gtkwidget), win_width,
+			      win_height);
+#endif
+#ifdef GTK1_ENABLED
+	    if (fs_window != NULL) {
+		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				GDK_WINDOW_XWINDOW(gtkwidget->window),
+				window, 0, 0);
+		XMapWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+			   GDK_WINDOW_XWINDOW(gtkwidget->window));
+		gtk_widget_destroy(fs_window);
+		fs_window = NULL;
+	    }
+
 	    XResizeWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 			  GDK_WINDOW_XWINDOW(gtkwidget->window),
 			  win_width, win_height);
-
+#endif
 	    if (panel_drawn == 1) {
 		height = 16;
 		width = 21;
@@ -1865,15 +2014,16 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 				       bg[GTK_STATE_NORMAL]));
 #endif
 #ifdef GTK1_ENABLED
-	    style = gtk_rc_style_new();
-	    style->bg[GTK_STATE_NORMAL] = color;
-	    style->color_flags[GTK_STATE_NORMAL] =
-		(GtkRcFlags) (style->
-			      color_flags[GTK_STATE_NORMAL] | GTK_RC_BG);
-	    gtk_widget_modify_style(gtkwidget, style);
-	    if (drawing_area != NULL)
-		gtk_widget_modify_style(drawing_area, style);
-	    gtk_rc_style_unref(style);
+// This style block does not seem to work anyway and since color is not set it could crash
+//          style = gtk_rc_style_new();
+//          style->bg[GTK_STATE_NORMAL] = color;
+//          style->color_flags[GTK_STATE_NORMAL] =
+//              (GtkRcFlags) (style->
+//                            color_flags[GTK_STATE_NORMAL] | GTK_RC_BG);
+//          gtk_widget_modify_style(gtkwidget, style);
+//          if (drawing_area != NULL)
+//              gtk_widget_modify_style(drawing_area, style);
+//          gtk_rc_style_unref(style);
 #endif
 #endif
 	    fullscreen = 0;
@@ -1883,24 +2033,48 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	    if (DEBUG > 1)
 		printf("SetFullscreen setting fullscreen\n");
 #ifdef GTK_ENABLED
+#ifdef GTK2_ENABLED
+	    fs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	    gtk_window_fullscreen(GTK_WINDOW(fs_window));
+	    gtk_widget_show(fs_window);
+	    gtk_widget_reparent(gtkwidget, fs_window);
 	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 			    GDK_WINDOW_XWINDOW(gtkwidget->window),
-			    GDK_ROOT_WINDOW(), 0, 0);
-	    // need to resize here
-#ifdef GTK2_ENABLED
-	    xdisp = GDK_WINDOW_XDISPLAY(gtkwidget->window);
-	    gdisp = gdk_x11_lookup_xdisplay(xdisp);
-	    screen = gdk_display_get_default_screen(gdisp);
-	    x = gdk_screen_get_width(screen);
-	    y = gdk_screen_get_height(screen);
+			    GDK_WINDOW_XWINDOW(fs_window->window), 0, 0);
+
+	    gtk_widget_map(gtkwidget);
+	    gdisp = gdk_display_get_default();
+	    screen = gdk_screen_get_default();
+	    gdk_screen_get_monitor_geometry(screen,
+					    gdk_screen_get_monitor_at_window
+					    (screen, gtkwidget->window),
+					    &rect);
+
+	    x = rect.width;
+	    y = rect.height;
+	    gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
+			      rect.height);
 #endif
 #ifdef GTK1_ENABLED
+	    fs_window = gtk_window_new(GTK_WINDOW_POPUP);
+	    gtk_widget_realize(fs_window);
+	    gtk_widget_show(fs_window);
+	    XReparentWindow(GDK_WINDOW_XDISPLAY(fs_window->window),
+			    GDK_WINDOW_XWINDOW(fs_window->window),
+			    GDK_ROOT_WINDOW(), 0, 0);
+	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+			    GDK_WINDOW_XWINDOW(gtkwidget->window),
+			    GDK_WINDOW_XWINDOW(fs_window->window), 0, 0);
+	    XMapWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+		       GDK_WINDOW_XWINDOW(gtkwidget->window));
 	    x = gdk_screen_width();
 	    y = gdk_screen_height();
-#endif
-
+	    XResizeWindow(GDK_WINDOW_XDISPLAY(fs_window->window),
+			  GDK_WINDOW_XWINDOW(fs_window->window), x, y);
 	    XResizeWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 			  GDK_WINDOW_XWINDOW(gtkwidget->window), x, y);
+#endif
+
 
 	    if (panel_drawn == 1) {
 		height = 16;
@@ -1944,21 +2118,28 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	    }
 
 	    disp_y = y;
+	    disp_x = x;
 	    if (movie_height != 0 && movie_width != 0) {
 		disp_y = ((long int) (x * movie_height)) / movie_width;
+		disp_x = ((long int) (y * movie_width)) / movie_height;
 	    } else {
-		if (win_width != 0)
+		if (win_width != 0) {
 		    disp_y = ((long int) (x * win_height)) / win_width;
+		    disp_x = ((long int) (y * win_width)) / win_height;
+		}
 	    }
 	    pos_y = (y - disp_y) / 2;
-	    if (disp_y > (y - 16)) {
-		disp_y = y - 16;
-		pos_y = 0;
-	    }
+	    pos_x = (x - disp_x) / 2;
 	    if (drawing_area != NULL) {
-		gtk_widget_set_usize(drawing_area, x, disp_y);
-		gtk_fixed_move(GTK_FIXED(fixed_container),
-			       GTK_WIDGET(drawing_area), 0, pos_y);
+		if (disp_y <= y) {
+		    gtk_widget_set_usize(drawing_area, x, disp_y);
+		    gtk_fixed_move(GTK_FIXED(fixed_container),
+				   GTK_WIDGET(drawing_area), 0, pos_y);
+		} else {
+		    gtk_widget_set_usize(drawing_area, disp_x, y);
+		    gtk_fixed_move(GTK_FIXED(fixed_container),
+				   GTK_WIDGET(drawing_area), pos_x, 0);
+		}
 	    }
 	    if (DEBUG > 1)
 		printf
