@@ -1,5 +1,5 @@
 #!/bin/bash
-if [ -f {HOME}/.mozilla/plugins/mplayerplug-in.* ]; then rm ${HOME}/.mozilla/plugins/mplayerplug-in.*; fi
+if [ -f ${HOME}/.mozilla/plugins/mplayerplug-in.so ]; then rm ${HOME}/.mozilla/plugins/mplayerplug-in.*; fi
 MOZ=`which mozilla`
 MOZHOME=`grep MOZILLA_FIVE_HOME= ${MOZ} | awk  -F \" '{ print $2 }'`
 if [ -z ${MOZHOME} ]; then MOZHOME=`grep moz_libdir= ${MOZ} | awk  -F \" '{ print $2 }'`; fi
